package javax.swing.text.html;

import javax.swing.event.DocumentEvent;
import java.awt.Shape;
import java.awt.Graphics;
import java.awt.Container;
import javax.swing.text.JTextComponent;
import java.util.Arrays;
import javax.swing.text.View;
import java.awt.Rectangle;
import javax.swing.text.StyleConstants;
import javax.swing.text.Element;
import java.util.BitSet;
import java.util.Vector;
import javax.swing.SizeRequirements;
import javax.swing.text.AttributeSet;
import javax.swing.text.ViewFactory;
import javax.swing.text.BoxView;

class TableView extends BoxView implements ViewFactory
{
    private AttributeSet attr;
    private StyleSheet.BoxPainter painter;
    private int cellSpacing;
    private int borderWidth;
    private int captionIndex;
    private boolean relativeCells;
    private boolean multiRowCells;
    int[] columnSpans;
    int[] columnOffsets;
    SizeRequirements totalColumnRequirements;
    SizeRequirements[] columnRequirements;
    TableView.RowIterator rowIterator;
    TableView.ColumnIterator colIterator;
    Vector<TableView.RowView> rows;
    boolean skipComments;
    boolean gridValid;
    private static final BitSet EMPTY;
    
    public TableView(Element elem) {
        super(elem, 1);
        this.rowIterator = new TableView.RowIterator();
        this.colIterator = new TableView.ColumnIterator();
        this.skipComments = false;
        this.rows = new Vector();
        this.gridValid = false;
        this.captionIndex = -1;
        this.totalColumnRequirements = new SizeRequirements();
    }
    
    protected TableView.RowView createTableRow(Element element) {
        if (element.getAttributes().getAttribute(StyleConstants.NameAttribute) == HTML.Tag.TR) {
            return new TableView.RowView(element);
        }
        return null;
    }
    
    public int getColumnCount() {
        return this.columnSpans.length;
    }
    
    public int getColumnSpan(int n) {
        if (n < this.columnSpans.length) {
            return this.columnSpans[n];
        }
        return 0;
    }
    
    public int getRowCount() {
        return this.rows.size();
    }
    
    public int getMultiRowSpan(int n, int n2) {
        TableView.RowView row = this.getRow(n);
        TableView.RowView row2 = this.getRow(n2);
        if (row != null && row2 != null) {
            int viewIndex = row.viewIndex;
            int viewIndex2 = row2.viewIndex;
            return this.getOffset(1, viewIndex2) - this.getOffset(1, viewIndex) + this.getSpan(1, viewIndex2);
        }
        return 0;
    }
    
    public int getRowSpan(int n) {
        TableView.RowView row = this.getRow(n);
        if (row != null) {
            return this.getSpan(1, row.viewIndex);
        }
        return 0;
    }
    
    TableView.RowView getRow(int index) {
        if (index < this.rows.size()) {
            return (TableView.RowView)this.rows.elementAt(index);
        }
        return null;
    }
    
    @Override
    protected View getViewAtPoint(int x, int y, Rectangle rectangle) {
        int viewCount = this.getViewCount();
        Rectangle rectangle2 = new Rectangle();
        for (int i = 0; i < viewCount; ++i) {
            rectangle2.setBounds(rectangle);
            this.childAllocation(i, rectangle2);
            View view = this.getView(i);
            if ((view instanceof TableView.RowView)) {
                View viewAtPoint = ((TableView.RowView)view).findViewAtPoint(x, y, rectangle2);
                if (viewAtPoint != null) {
                    rectangle.setBounds(rectangle2);
                    return viewAtPoint;
                }
            }
        }
        return super.getViewAtPoint(x, y, rectangle);
    }
    
    protected int getColumnsOccupied(View view) {
        AttributeSet attributes = view.getElement().getAttributes();
        if (attributes.isDefined(HTML.Attribute.COLSPAN)) {
            String s = (String)attributes.getAttribute(HTML.Attribute.COLSPAN);
            if (s != null) {
                try {
                    return Integer.parseInt(s);
                }
                catch (NumberFormatException ex) {}
            }
        }
        return 1;
    }
    
    protected int getRowsOccupied(View view) {
        AttributeSet attributes = view.getElement().getAttributes();
        if (attributes.isDefined(HTML.Attribute.ROWSPAN)) {
            String s = (String)attributes.getAttribute(HTML.Attribute.ROWSPAN);
            if (s != null) {
                try {
                    return Integer.parseInt(s);
                }
                catch (NumberFormatException ex) {}
            }
        }
        return 1;
    }
    
    protected void invalidateGrid() {
        this.gridValid = false;
    }
    
    protected StyleSheet getStyleSheet() {
        return ((HTMLDocument)this.getDocument()).getStyleSheet();
    }
    
    void updateInsets() {
        int top = (short)(int)this.painter.getInset(1, this);
        int bottom = (short)(int)this.painter.getInset(3, this);
        if (this.captionIndex != -1) {
            View view = this.getView(this.captionIndex);
            int n = (short)(int)view.getPreferredSpan(1);
            Object attribute = view.getAttributes().getAttribute(CSS.Attribute.CAPTION_SIDE);
            if (attribute != null && attribute.equals("bottom")) {
                bottom = (short)(bottom + n);
            }
            else {
                top = (short)(top + n);
            }
        }
        this.setInsets((short)top, (short)(int)this.painter.getInset(2, this), (short)bottom, (short)(int)this.painter.getInset(4, this));
    }
    
    protected void setPropertiesFromAttributes() {
        StyleSheet styleSheet = this.getStyleSheet();
        this.attr = styleSheet.getViewAttributes(this);
        this.painter = styleSheet.getBoxPainter(this.attr);
        if (this.attr != null) {
            this.setInsets((short)(int)this.painter.getInset(1, this), (short)(int)this.painter.getInset(2, this), (short)(int)this.painter.getInset(3, this), (short)(int)this.painter.getInset(4, this));
            CSS.LengthValue lengthValue = (CSS.LengthValue)this.attr.getAttribute(CSS.Attribute.BORDER_SPACING);
            if (lengthValue != null) {
                this.cellSpacing = (int)lengthValue.getValue();
            }
            else {
                this.cellSpacing = 2;
            }
            CSS.LengthValue lengthValue2 = (CSS.LengthValue)this.attr.getAttribute(CSS.Attribute.BORDER_TOP_WIDTH);
            if (lengthValue2 != null) {
                this.borderWidth = (int)lengthValue2.getValue();
            }
            else {
                this.borderWidth = 0;
            }
        }
    }
    
    void updateGrid() {
        if (!this.gridValid) {
            this.relativeCells = false;
            this.multiRowCells = false;
            this.captionIndex = -1;
            this.rows.removeAllElements();
            for (int viewCount = this.getViewCount(), i = 0; i < viewCount; ++i) {
                View view = this.getView(i);
                if ((view instanceof TableView.RowView)) {
                    this.rows.addElement((TableView.RowView)view);
                    TableView.RowView rowView = (TableView.RowView)view;
                    rowView.clearFilledColumns();
                    rowView.rowIndex = this.rows.size() - 1;
                    rowView.viewIndex = i;
                }
                else {
                    Object attribute = view.getElement().getAttributes().getAttribute(StyleConstants.NameAttribute);
                    if (attribute instanceof HTML.Tag && (HTML.Tag)attribute == HTML.Tag.CAPTION) {
                        this.captionIndex = i;
                    }
                }
            }
            int max = 0;
            int k;
            for (int size = this.rows.size(), j = 0; j < size; max = Math.max(max, k), ++j) {
                TableView.RowView row = this.getRow(j);
                k = 0;
                for (int l = 0; l < row.getViewCount(); ++l, ++k) {
                    View view2 = row.getView(l);
                    if (!this.relativeCells) {
                        CSS.LengthValue lengthValue = (CSS.LengthValue)view2.getAttributes().getAttribute(CSS.Attribute.WIDTH);
                        if (lengthValue != null && lengthValue.isPercentage()) {
                            this.relativeCells = true;
                        }
                    }
                    while (row.isFilled(k)) {
                        ++k;
                    }
                    int rowsOccupied = this.getRowsOccupied(view2);
                    if (rowsOccupied > 1) {
                        this.multiRowCells = true;
                    }
                    int columnsOccupied = this.getColumnsOccupied(view2);
                    if (columnsOccupied > 1 || rowsOccupied > 1) {
                        int n = j + rowsOccupied;
                        int n2 = k + columnsOccupied;
                        for (; j < n; ++j) {
                            for (; k < n2; ++k) {
                                if (j != j || k != k) {
                                    this.addFill(j, k);
                                }
                            }
                        }
                        if (columnsOccupied > 1) {
                            k += columnsOccupied - 1;
                        }
                    }
                }
            }
            this.columnSpans = new int[max];
            this.columnOffsets = new int[max];
            this.columnRequirements = new SizeRequirements[max];
            for (int n3 = 0; n3 < max; ++n3) {
                this.columnRequirements[n3] = new SizeRequirements();
                this.columnRequirements[n3].maximum = Integer.MAX_VALUE;
            }
            this.gridValid = true;
        }
    }
    
    void addFill(int n, int n2) {
        TableView.RowView row = this.getRow(n);
        if (row != null) {
            row.fillColumn(n2);
        }
    }
    
    protected void layoutColumns(int n, int[] a, int[] a2, SizeRequirements[] array) {
        Arrays.fill(a, 0);
        Arrays.fill(a2, 0);
        this.colIterator.setLayoutArrays(a, a2, n);
        CSS.calculateTiledLayout(this.colIterator, n);
    }
    
    void calculateColumnRequirements(int n) {
        for (SizeRequirements sizeRequirements : this.columnRequirements) {
            sizeRequirements.minimum = 0;
            sizeRequirements.preferred = 0;
            sizeRequirements.maximum = Integer.MAX_VALUE;
        }
        Container container = this.getContainer();
        if (container != null) {
            if ((container instanceof JTextComponent)) {
                this.skipComments = !((JTextComponent)container).isEditable();
            }
            else {
                this.skipComments = true;
            }
        }
        boolean n2 = false;
        int rowCount = this.getRowCount();
        for (int j = 0; j < rowCount; ++j) {
            TableView.RowView row = this.getRow(j);
            int n3 = 0;
            for (int viewCount = row.getViewCount(), k = 0; k < viewCount; ++k) {
                View view = row.getView(k);
                if (!this.skipComments || view instanceof TableView.CellView) {
                    while (row.isFilled(n3)) {
                        ++n3;
                    }
                    this.getRowsOccupied(view);
                    int columnsOccupied = this.getColumnsOccupied(view);
                    if (columnsOccupied == 1) {
                        this.checkSingleColumnCell(n, n3, view);
                    }
                    else {
                        n2 = true;
                        n3 += columnsOccupied - 1;
                    }
                    ++n3;
                }
            }
        }
        if (!(n2 == 0)) {
            for (int l = 0; l < rowCount; ++l) {
                TableView.RowView row2 = this.getRow(l);
                int n4 = 0;
                for (int viewCount2 = row2.getViewCount(), n5 = 0; n5 < viewCount2; ++n5) {
                    View view2 = row2.getView(n5);
                    if (!this.skipComments || view2 instanceof TableView.CellView) {
                        while (row2.isFilled(n4)) {
                            ++n4;
                        }
                        int columnsOccupied2 = this.getColumnsOccupied(view2);
                        if (columnsOccupied2 > 1) {
                            this.checkMultiColumnCell(n, n4, columnsOccupied2, view2);
                            n4 += columnsOccupied2 - 1;
                        }
                        ++n4;
                    }
                }
            }
        }
    }
    
    void checkSingleColumnCell(int axis, int n, View view) {
        SizeRequirements sizeRequirements = this.columnRequirements[n];
        sizeRequirements.minimum = Math.max((int)view.getMinimumSpan(axis), sizeRequirements.minimum);
        sizeRequirements.preferred = Math.max((int)view.getPreferredSpan(axis), sizeRequirements.preferred);
    }
    
    void checkMultiColumnCell(int axis, int n, int n2, View view) {
        long n3 = 0L;
        long n4 = 0L;
        long n5 = 0L;
        for (int i = 0; i < n2; ++i) {
            SizeRequirements sizeRequirements = this.columnRequirements[n + i];
            n3 += (long)sizeRequirements.minimum;
            n4 += (long)sizeRequirements.preferred;
            n5 += (long)sizeRequirements.maximum;
        }
        int allocated = (int)view.getMinimumSpan(axis);
        if ((long)allocated > n3) {
            SizeRequirements[] children = new SizeRequirements[n2];
            for (int j = 0; j < n2; ++j) {
                children[j] = this.columnRequirements[n + j];
            }
            int[] spans = new int[n2];
            SizeRequirements.calculateTiledPositions(allocated, null, children, new int[n2], spans);
            for (int k = 0; k < n2; ++k) {
                SizeRequirements sizeRequirements2 = children[k];
                sizeRequirements2.minimum = Math.max(spans[k], sizeRequirements2.minimum);
                sizeRequirements2.preferred = Math.max(sizeRequirements2.minimum, sizeRequirements2.preferred);
                sizeRequirements2.maximum = Math.max(sizeRequirements2.preferred, sizeRequirements2.maximum);
            }
        }
        int allocated2 = (int)view.getPreferredSpan(axis);
        if ((long)allocated2 > n4) {
            SizeRequirements[] children2 = new SizeRequirements[n2];
            for (int l = 0; l < n2; ++l) {
                children2[l] = this.columnRequirements[n + l];
            }
            int[] spans2 = new int[n2];
            SizeRequirements.calculateTiledPositions(allocated2, null, children2, new int[n2], spans2);
            for (int n6 = 0; n6 < n2; ++n6) {
                SizeRequirements sizeRequirements3 = children2[n6];
                sizeRequirements3.preferred = Math.max(spans2[n6], sizeRequirements3.preferred);
                sizeRequirements3.maximum = Math.max(sizeRequirements3.preferred, sizeRequirements3.maximum);
            }
        }
    }
    
    @Override
    protected SizeRequirements calculateMinorAxisRequirements(int n, SizeRequirements sizeRequirements) {
        this.updateGrid();
        this.calculateColumnRequirements(n);
        if (sizeRequirements == null) {
            sizeRequirements = new SizeRequirements();
        }
        long n2 = 0L;
        long n3 = 0L;
        int length = this.columnRequirements.length;
        for (int i = 0; i < length; ++i) {
            SizeRequirements sizeRequirements2 = this.columnRequirements[i];
            n2 += (long)sizeRequirements2.minimum;
            n3 += (long)sizeRequirements2.preferred;
        }
        int n4 = (length + 1) * this.cellSpacing + 2 * this.borderWidth;
        long n5 = n2 + (long)n4;
        long n6 = n3 + (long)n4;
        sizeRequirements.minimum = (int)n5;
        sizeRequirements.preferred = (int)n6;
        sizeRequirements.maximum = (int)n6;
        AttributeSet attributes = this.getAttributes();
        if (BlockView.spanSetFromAttributes(n, sizeRequirements, (CSS.LengthValue)attributes.getAttribute(CSS.Attribute.WIDTH), null) && sizeRequirements.minimum < (int)n5) {
            SizeRequirements sizeRequirements3 = sizeRequirements;
            SizeRequirements sizeRequirements4 = sizeRequirements;
            SizeRequirements sizeRequirements5 = sizeRequirements;
            int maximum = (int)n5;
            sizeRequirements5.preferred = maximum;
            sizeRequirements4.minimum = maximum;
            sizeRequirements3.maximum = maximum;
        }
        this.totalColumnRequirements.minimum = sizeRequirements.minimum;
        this.totalColumnRequirements.preferred = sizeRequirements.preferred;
        this.totalColumnRequirements.maximum = sizeRequirements.maximum;
        Object attribute = attributes.getAttribute(CSS.Attribute.TEXT_ALIGN);
        if (attribute != null) {
            String string = attribute.toString();
            if (string.equals("left")) {
                sizeRequirements.alignment = 0.0f;
            }
            else if (string.equals("center")) {
                sizeRequirements.alignment = 0.5f;
            }
            else if (string.equals("right")) {
                sizeRequirements.alignment = 1.0f;
            }
            else {
                sizeRequirements.alignment = 0.0f;
            }
            return sizeRequirements;
        }
        else {
            sizeRequirements.alignment = 0.0f;
            return sizeRequirements;
        }
    }
    
    @Override
    protected SizeRequirements calculateMajorAxisRequirements(int n, SizeRequirements calculateTiledRequirements) {
        this.updateInsets();
        this.rowIterator.updateAdjustments();
        calculateTiledRequirements = CSS.calculateTiledRequirements(this.rowIterator, calculateTiledRequirements);
        calculateTiledRequirements.maximum = calculateTiledRequirements.preferred;
        return calculateTiledRequirements;
    }
    
    @Override
    protected void layoutMinorAxis(int targetSpan, int n, int[] offsets, int[] spans) {
        this.updateGrid();
        for (int rowCount = this.getRowCount(), i = 0; i < rowCount; ++i) {
            this.getRow(i).layoutChanged(n);
        }
        this.layoutColumns(targetSpan, this.columnOffsets, this.columnSpans, this.columnRequirements);
        super.layoutMinorAxis(targetSpan, n, offsets, spans);
    }
    
    @Override
    protected void layoutMajorAxis(int n, int n2, int[] array, int[] array2) {
        this.rowIterator.setLayoutArrays(array, array2);
        CSS.calculateTiledLayout(this.rowIterator, n);
        if (this.captionIndex != -1) {
            array2[this.captionIndex] = (int)this.getView(this.captionIndex).getPreferredSpan(1);
            int n3 = (short)(int)this.painter.getInset(3, this);
            if (n3 != this.getBottomInset()) {
                array[this.captionIndex] = n + n3;
            }
            else {
                array[this.captionIndex] = -this.getTopInset();
            }
        }
    }
    
    @Override
    protected View getViewAtPosition(int n, Rectangle rectangle) {
        int viewCount = this.getViewCount();
        int i = 0;
        while (i < viewCount) {
            View view = this.getView(i);
            int startOffset = view.getStartOffset();
            int endOffset = view.getEndOffset();
            if (n >= startOffset && n < endOffset) {
                if (rectangle != null) {
                    this.childAllocation(i, rectangle);
                    return view;
                }
                return view;
            }
            else {
                ++i;
            }
        }
        if (n == this.getEndOffset()) {
            View view2 = this.getView(viewCount - 1);
            if (rectangle != null) {
                this.childAllocation(viewCount - 1, rectangle);
                return view2;
            }
            return view2;
        }
        else {
            return null;
        }
    }
    
    @Override
    public AttributeSet getAttributes() {
        if (this.attr == null) {
            this.attr = this.getStyleSheet().getViewAttributes(this);
        }
        return this.attr;
    }
    
    @Override
    public void paint(Graphics graphics, Shape a) {
        Rectangle bounds = a.getBounds();
        this.setSize((float)bounds.width, (float)bounds.height);
        if (this.captionIndex != -1) {
            int n = (short)(int)this.painter.getInset(1, this);
            int n2 = (short)(int)this.painter.getInset(3, this);
            if (n != this.getTopInset()) {
                int n3 = this.getTopInset() - n;
                bounds.y += n3;
                bounds.height -= n3;
            }
            else {
                bounds.height -= this.getBottomInset() - n2;
            }
        }
        this.painter.paint(graphics, (float)bounds.x, (float)bounds.y, (float)bounds.width, (float)bounds.height, this);
        for (int viewCount = this.getViewCount(), i = 0; i < viewCount; ++i) {
            this.getView(i).paint(graphics, this.getChildAllocation(i, a));
        }
    }
    
    @Override
    public void setParent(View parent) {
        super.setParent(parent);
        if (parent != null) {
            this.setPropertiesFromAttributes();
        }
    }
    
    @Override
    public ViewFactory getViewFactory() {
        return this;
    }
    
    @Override
    public void insertUpdate(DocumentEvent e, Shape a, ViewFactory viewFactory) {
        super.insertUpdate(e, a, this);
    }
    
    @Override
    public void removeUpdate(DocumentEvent e, Shape a, ViewFactory viewFactory) {
        super.removeUpdate(e, a, this);
    }
    
    @Override
    public void changedUpdate(DocumentEvent e, Shape a, ViewFactory viewFactory) {
        super.changedUpdate(e, a, this);
    }
    
    @Override
    protected void forwardUpdate(DocumentEvent.ElementChange ec, DocumentEvent e, Shape a, ViewFactory f) {
        super.forwardUpdate(ec, e, a, f);
        if (a != null) {
            Container container = this.getContainer();
            if (container != null) {
                Rectangle rectangle = (a instanceof Rectangle) ? ((Rectangle)a) : a.getBounds();
                container.repaint(rectangle.x, rectangle.y, rectangle.width, rectangle.height);
            }
        }
    }
    
    @Override
    public void replace(int index, int length, View[] elems) {
        super.replace(index, length, elems);
        this.invalidateGrid();
    }
    
    @Override
    public View create(Element element) {
        Object attribute = element.getAttributes().getAttribute(StyleConstants.NameAttribute);
        if ((attribute instanceof HTML.Tag)) {
            HTML.Tag tag = (HTML.Tag)attribute;
            if (tag == HTML.Tag.TR) {
                return this.createTableRow(element);
            }
            if (tag == HTML.Tag.TD || tag == HTML.Tag.TH) {
                return new TableView.CellView(element);
            }
            if (tag == HTML.Tag.CAPTION) {
                return new ParagraphView(element);
            }
        }
        View parent = this.getParent();
        if (parent != null) {
            ViewFactory viewFactory = parent.getViewFactory();
            if (viewFactory != null) {
                return viewFactory.create(element);
            }
        }
        return null;
    }
    
    static {
        EMPTY = new BitSet();
    }
    
    class ColumnIterator implements CSS.LayoutIterator
    {
        private int col;
        private int[] percentages;
        private int[] adjustmentWeights;
        private int[] offsets;
        private int[] spans;
        
        void disablePercentages() {
            this.percentages = null;
        }
        
        private void updatePercentagesAndAdjustmentWeights(int n) {
            this.adjustmentWeights = new int[TableView.this.columnRequirements.length];
            for (int i = 0; i < TableView.this.columnRequirements.length; ++i) {
                this.adjustmentWeights[i] = 0;
            }
            if (TableView.this.relativeCells) {
                this.percentages = new int[TableView.this.columnRequirements.length];
            }
            else {
                this.percentages = null;
            }
            for (int rowCount = TableView.this.getRowCount(), j = 0; j < rowCount; ++j) {
                TableView.RowView row = TableView.this.getRow(j);
                int columnsOccupied;
                for (int n2 = 0, viewCount = row.getViewCount(), k = 0; k < viewCount; n2 += columnsOccupied - 1, ++k, ++n2) {
                    View view = row.getView(k);
                    while (row.isFilled(n2)) {
                        ++n2;
                    }
                    TableView.this.getRowsOccupied(view);
                    columnsOccupied = TableView.this.getColumnsOccupied(view);
                    CSS.LengthValue lengthValue = (CSS.LengthValue)view.getAttributes().getAttribute(CSS.Attribute.WIDTH);
                    if (lengthValue != null) {
                        int b = (int)(lengthValue.getValue((float)n) / (float)columnsOccupied + 0.5f);
                        for (int l = 0; l < columnsOccupied; ++l) {
                            if (lengthValue.isPercentage()) {
                                this.percentages[n2 + l] = Math.max(this.percentages[n2 + l], b);
                                this.adjustmentWeights[n2 + l] = Math.max(this.adjustmentWeights[n2 + l], 2);
                            }
                            else {
                                this.adjustmentWeights[n2 + l] = Math.max(this.adjustmentWeights[n2 + l], 1);
                            }
                        }
                    }
                }
            }
        }
        
        public void setLayoutArrays(int[] offsets, int[] spans, int n) {
            this.offsets = offsets;
            this.spans = spans;
            this.updatePercentagesAndAdjustmentWeights(n);
        }
        
        @Override
        public int getCount() {
            return TableView.this.columnRequirements.length;
        }
        
        @Override
        public void setIndex(int col) {
            this.col = col;
        }
        
        @Override
        public void setOffset(int n) {
            this.offsets[this.col] = n;
        }
        
        @Override
        public int getOffset() {
            return this.offsets[this.col];
        }
        
        @Override
        public void setSpan(int n) {
            this.spans[this.col] = n;
        }
        
        @Override
        public int getSpan() {
            return this.spans[this.col];
        }
        
        @Override
        public float getMinimumSpan(float n) {
            return (float)TableView.this.columnRequirements[this.col].minimum;
        }
        
        @Override
        public float getPreferredSpan(float n) {
            if (this.percentages != null && this.percentages[this.col] != 0) {
                return (float)Math.max(this.percentages[this.col], TableView.this.columnRequirements[this.col].minimum);
            }
            return (float)TableView.this.columnRequirements[this.col].preferred;
        }
        
        @Override
        public float getMaximumSpan(float n) {
            return (float)TableView.this.columnRequirements[this.col].maximum;
        }
        
        @Override
        public float getBorderWidth() {
            return (float)TableView.this.borderWidth;
        }
        
        @Override
        public float getLeadingCollapseSpan() {
            return (float)TableView.this.cellSpacing;
        }
        
        @Override
        public float getTrailingCollapseSpan() {
            return (float)TableView.this.cellSpacing;
        }
        
        @Override
        public int getAdjustmentWeight() {
            return this.adjustmentWeights[this.col];
        }
    }
    
    class RowIterator implements CSS.LayoutIterator
    {
        private int row;
        private int[] adjustments;
        private int[] offsets;
        private int[] spans;
        
        void updateAdjustments() {
            int n = 1;
            if (TableView.this.multiRowCells) {
                int rowCount = TableView.this.getRowCount();
                this.adjustments = new int[rowCount];
                for (int i = 0; i < rowCount; ++i) {
                    TableView.RowView row = TableView.this.getRow(i);
                    if (row.multiRowCells) {
                        for (int viewCount = row.getViewCount(), j = 0; j < viewCount; ++j) {
                            View view = row.getView(j);
                            int rowsOccupied = TableView.this.getRowsOccupied(view);
                            if (rowsOccupied > 1) {
                                this.adjustMultiRowSpan((int)view.getPreferredSpan(n), rowsOccupied, i);
                            }
                        }
                    }
                }
            }
            else {
                this.adjustments = null;
            }
        }
        
        void adjustMultiRowSpan(int n, int n2, int n3) {
            if (n3 + n2 > this.getCount()) {
                n2 = this.getCount() - n3;
                if (n2 < 1) {
                    return;
                }
            }
            int n4 = 0;
            for (int i = 0; i < n2; ++i) {
                n4 = (int)((float)n4 + TableView.this.getRow(n3 + i).getPreferredSpan(1));
            }
            if (n > n4) {
                int n5 = n - n4;
                int b = n5 / n2;
                int b2 = b + (n5 - b * n2);
                TableView.this.getRow(n3);
                this.adjustments[n3] = Math.max(this.adjustments[n3], b2);
                for (int j = 1; j < n2; ++j) {
                    this.adjustments[n3 + j] = Math.max(this.adjustments[n3 + j], b);
                }
            }
        }
        
        void setLayoutArrays(int[] offsets, int[] spans) {
            this.offsets = offsets;
            this.spans = spans;
        }
        
        @Override
        public void setOffset(int n) {
            TableView.RowView row = TableView.this.getRow(this.row);
            if (row != null) {
                this.offsets[row.viewIndex] = n;
            }
        }
        
        @Override
        public int getOffset() {
            TableView.RowView row = TableView.this.getRow(this.row);
            if (row != null) {
                return this.offsets[row.viewIndex];
            }
            return 0;
        }
        
        @Override
        public void setSpan(int n) {
            TableView.RowView row = TableView.this.getRow(this.row);
            if (row != null) {
                this.spans[row.viewIndex] = n;
            }
        }
        
        @Override
        public int getSpan() {
            TableView.RowView row = TableView.this.getRow(this.row);
            if (row != null) {
                return this.spans[row.viewIndex];
            }
            return 0;
        }
        
        @Override
        public int getCount() {
            return TableView.this.rows.size();
        }
        
        @Override
        public void setIndex(int row) {
            this.row = row;
        }
        
        @Override
        public float getMinimumSpan(float n) {
            return this.getPreferredSpan(n);
        }
        
        @Override
        public float getPreferredSpan(float n) {
            TableView.RowView row = TableView.this.getRow(this.row);
            if (row != null) {
                return row.getPreferredSpan(TableView.this.getAxis()) + (float)((this.adjustments != null) ? this.adjustments[this.row] : 0);
            }
            return 0.0f;
        }
        
        @Override
        public float getMaximumSpan(float n) {
            return this.getPreferredSpan(n);
        }
        
        @Override
        public float getBorderWidth() {
            return (float)TableView.this.borderWidth;
        }
        
        @Override
        public float getLeadingCollapseSpan() {
            return (float)TableView.this.cellSpacing;
        }
        
        @Override
        public float getTrailingCollapseSpan() {
            return (float)TableView.this.cellSpacing;
        }
        
        @Override
        public int getAdjustmentWeight() {
            return 0;
        }
    }
    
    public class RowView extends BoxView
    {
        private StyleSheet.BoxPainter painter;
        private AttributeSet attr;
        BitSet fillColumns;
        int rowIndex;
        int viewIndex;
        boolean multiRowCells;
        
        public RowView(Element elem) {
            super(elem, 0);
            this.fillColumns = new BitSet();
            this.setPropertiesFromAttributes();
        }
        
        void clearFilledColumns() {
            this.fillColumns.and(TableView.EMPTY);
        }
        
        void fillColumn(int bitIndex) {
            this.fillColumns.set(bitIndex);
        }
        
        boolean isFilled(int bitIndex) {
            return this.fillColumns.get(bitIndex);
        }
        
        int getColumnCount() {
            int n = 0;
            for (int size = this.fillColumns.size(), i = 0; i < size; ++i) {
                if (this.fillColumns.get(i)) {
                    ++n;
                }
            }
            return this.getViewCount() + n;
        }
        
        @Override
        public AttributeSet getAttributes() {
            return this.attr;
        }
        
        View findViewAtPoint(int n, int n2, Rectangle rectangle) {
            int viewCount = this.getViewCount();
            int i = 0;
            while (i < viewCount) {
                if (this.getChildAllocation(i, rectangle).contains((double)n, (double)n2)) {
                    this.childAllocation(i, rectangle);
                    return this.getView(i);
                }
                ++i;
            }
            return null;
        }
        
        protected StyleSheet getStyleSheet() {
            return ((HTMLDocument)this.getDocument()).getStyleSheet();
        }
        
        @Override
        public void preferenceChanged(View child, boolean width, boolean height) {
            super.preferenceChanged(child, width, height);
            if (TableView.this.multiRowCells && height) {
                int i = this.rowIndex - 1;
                while (i >= 0) {
                    TableView.RowView row = TableView.this.getRow(i);
                    if (row.multiRowCells) {
                        row.preferenceChanged(null, false, true);
                        break;
                    }
                    --i;
                }
            }
        }
        
        @Override
        protected SizeRequirements calculateMajorAxisRequirements(int n, SizeRequirements sizeRequirements) {
            SizeRequirements sizeRequirements2 = new SizeRequirements();
            sizeRequirements2.minimum = TableView.this.totalColumnRequirements.minimum;
            sizeRequirements2.maximum = TableView.this.totalColumnRequirements.maximum;
            sizeRequirements2.preferred = TableView.this.totalColumnRequirements.preferred;
            sizeRequirements2.alignment = 0.0f;
            return sizeRequirements2;
        }
        
        @Override
        public float getMinimumSpan(int axis) {
            float minimumSpan;
            if (axis == 0) {
                minimumSpan = (float)(TableView.this.totalColumnRequirements.minimum + this.getLeftInset() + this.getRightInset());
            }
            else {
                minimumSpan = super.getMinimumSpan(axis);
            }
            return minimumSpan;
        }
        
        @Override
        public float getMaximumSpan(int axis) {
            float maximumSpan;
            if (axis == 0) {
                maximumSpan = 2.1474836E9f;
            }
            else {
                maximumSpan = super.getMaximumSpan(axis);
            }
            return maximumSpan;
        }
        
        @Override
        public float getPreferredSpan(int axis) {
            float preferredSpan;
            if (axis == 0) {
                preferredSpan = (float)(TableView.this.totalColumnRequirements.preferred + this.getLeftInset() + this.getRightInset());
            }
            else {
                preferredSpan = super.getPreferredSpan(axis);
            }
            return preferredSpan;
        }
        
        @Override
        public void changedUpdate(DocumentEvent e, Shape a, ViewFactory f) {
            super.changedUpdate(e, a, f);
            int offset = e.getOffset();
            if (offset <= this.getStartOffset() && offset + e.getLength() >= this.getEndOffset()) {
                this.setPropertiesFromAttributes();
            }
        }
        
        @Override
        public void paint(Graphics g, Shape shape) {
            Rectangle allocation = (Rectangle)shape;
            this.painter.paint(g, (float)allocation.x, (float)allocation.y, (float)allocation.width, (float)allocation.height, this);
            super.paint(g, allocation);
        }
        
        @Override
        public void replace(int index, int length, View[] elems) {
            super.replace(index, length, elems);
            TableView.this.invalidateGrid();
        }
        
        @Override
        protected SizeRequirements calculateMinorAxisRequirements(int axis, SizeRequirements sizeRequirements) {
            long max = 0L;
            long max2 = 0L;
            long n = 0L;
            this.multiRowCells = false;
            for (int viewCount = this.getViewCount(), i = 0; i < viewCount; ++i) {
                View view = this.getView(i);
                if (TableView.this.getRowsOccupied(view) > 1) {
                    this.multiRowCells = true;
                    n = Math.max((long)(int)view.getMaximumSpan(axis), n);
                }
                else {
                    max = Math.max((long)(int)view.getMinimumSpan(axis), max);
                    max2 = Math.max((long)(int)view.getPreferredSpan(axis), max2);
                    n = Math.max((long)(int)view.getMaximumSpan(axis), n);
                }
            }
            if (sizeRequirements == null) {
                sizeRequirements = new SizeRequirements();
                sizeRequirements.alignment = 0.5f;
            }
            sizeRequirements.preferred = (int)max2;
            sizeRequirements.minimum = (int)max;
            sizeRequirements.maximum = (int)n;
            return sizeRequirements;
        }
        
        @Override
        protected void layoutMajorAxis(int n, int n2, int[] array, int[] array2) {
            int n3 = 0;
            for (int viewCount = this.getViewCount(), i = 0; i < viewCount; ++i) {
                View view = this.getView(i);
                if (!TableView.this.skipComments || view instanceof TableView.CellView) {
                    while (this.isFilled(n3)) {
                        ++n3;
                    }
                    int columnsOccupied = TableView.this.getColumnsOccupied(view);
                    array2[i] = TableView.this.columnSpans[n3];
                    array[i] = TableView.this.columnOffsets[n3];
                    if (columnsOccupied > 1) {
                        int length = TableView.this.columnSpans.length;
                        for (int j = 1; j < columnsOccupied; ++j) {
                            if (n3 + j < length) {
                                int n4 = i;
                                array2[n4] += TableView.this.columnSpans[n3 + j];
                                int n5 = i;
                                array2[n5] += TableView.this.cellSpacing;
                            }
                        }
                        n3 += columnsOccupied - 1;
                    }
                    ++n3;
                }
            }
        }
        
        @Override
        protected void layoutMinorAxis(int targetSpan, int axis, int[] offsets, int[] spans) {
            super.layoutMinorAxis(targetSpan, axis, offsets, spans);
            for (int n = 0, viewCount = this.getViewCount(), i = 0; i < viewCount; ++i, ++n) {
                View view = this.getView(i);
                while (this.isFilled(n)) {
                    ++n;
                }
                int columnsOccupied = TableView.this.getColumnsOccupied(view);
                int rowsOccupied = TableView.this.getRowsOccupied(view);
                if (rowsOccupied > 1) {
                    spans[i] = TableView.this.getMultiRowSpan(this.rowIndex, Math.min(this.rowIndex + rowsOccupied - 1, TableView.this.getRowCount() - 1));
                }
                if (columnsOccupied > 1) {
                    n += columnsOccupied - 1;
                }
            }
        }
        
        @Override
        public int getResizeWeight(int n) {
            return 1;
        }
        
        @Override
        protected View getViewAtPosition(int n, Rectangle rectangle) {
            int viewCount = this.getViewCount();
            int i = 0;
            while (i < viewCount) {
                View view = this.getView(i);
                int startOffset = view.getStartOffset();
                int endOffset = view.getEndOffset();
                if (n >= startOffset && n < endOffset) {
                    if (rectangle != null) {
                        this.childAllocation(i, rectangle);
                        return view;
                    }
                    return view;
                }
                else {
                    ++i;
                }
            }
            if (n == this.getEndOffset()) {
                View view2 = this.getView(viewCount - 1);
                if (rectangle != null) {
                    this.childAllocation(viewCount - 1, rectangle);
                    return view2;
                }
                return view2;
            }
            else {
                return null;
            }
        }
        
        void setPropertiesFromAttributes() {
            StyleSheet styleSheet = this.getStyleSheet();
            this.attr = styleSheet.getViewAttributes(this);
            this.painter = styleSheet.getBoxPainter(this.attr);
        }
    }
    
    class CellView extends BlockView
    {
        public CellView(Element element) {
            super(element, 1);
        }
        
        @Override
        protected void layoutMajorAxis(int targetSpan, int axis, int[] offsets, int[] spans) {
            super.layoutMajorAxis(targetSpan, axis, offsets, spans);
            int n = 0;
            int length = spans.length;
            for (int i = 0; i < length; ++i) {
                n += spans[i];
            }
            int n2 = 0;
            if (n < targetSpan) {
                String s = (String)this.getElement().getAttributes().getAttribute(HTML.Attribute.VALIGN);
                if (s == null) {
                    s = (String)this.getElement().getParentElement().getAttributes().getAttribute(HTML.Attribute.VALIGN);
                }
                if (s == null || s.equals("middle")) {
                    n2 = (targetSpan - n) / 2;
                }
                else if (s.equals("bottom")) {
                    n2 = targetSpan - n;
                }
            }
            if (n2 != 0) {
                for (int j = 0; j < length; ++j) {
                    int n3 = j;
                    offsets[n3] += n2;
                }
            }
        }
        
        @Override
        protected SizeRequirements calculateMajorAxisRequirements(int n, SizeRequirements sizeRequirements) {
            SizeRequirements calculateMajorAxisRequirements = super.calculateMajorAxisRequirements(n, sizeRequirements);
            calculateMajorAxisRequirements.maximum = Integer.MAX_VALUE;
            return calculateMajorAxisRequirements;
        }
        
        @Override
        protected SizeRequirements calculateMinorAxisRequirements(int axis, SizeRequirements sizeRequirements) {
            SizeRequirements calculateMinorAxisRequirements = super.calculateMinorAxisRequirements(axis, sizeRequirements);
            int viewCount = this.getViewCount();
            int max = 0;
            for (int i = 0; i < viewCount; ++i) {
                max = Math.max((int)this.getView(i).getMinimumSpan(axis), max);
            }
            calculateMinorAxisRequirements.minimum = Math.min(calculateMinorAxisRequirements.minimum, max);
            return calculateMinorAxisRequirements;
        }
    }
}
